@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999-2007, 2009-2012 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <frm.h>
 
@@ -62,13 +59,9 @@ static char *output_charset = NULL;
 const char *
 get_charset ()
 {
-  char *tmp;
-  
   if (!output_charset)
     {
-      char locale[32];
-      
-      memset (locale, 0, sizeof (locale));
+      char *tmp;
 
       /* Try to deduce the charset from LC_ALL or LANG variables */
 
@@ -78,24 +71,14 @@ get_charset ()
 
       if (tmp)
 	{
-	  char *sp = NULL;
-	  char *lang;
-	  char *terr;
-
-	  strncpy (locale, tmp, sizeof (locale) - 1);
+	  struct mu_lc_all lc_all;
 	  
-	  lang = strtok_r (locale, "_", &sp);
-	  terr = strtok_r (NULL, ".", &sp);
-	  output_charset = strtok_r (NULL, "@", &sp);
-
-	  if (output_charset)
-	    output_charset = xstrdup (output_charset);
-	  else
-	    output_charset = mu_charset_lookup (lang, terr);
+	  if (mu_parse_lc_all (tmp, &lc_all, MU_LC_CSET) == 0)
+	    output_charset = lc_all.charset;
 	}
       
       if (!output_charset)
-	output_charset = "ASCII";
+	output_charset = xstrdup ("ASCII");
     }
   return output_charset;
 }
@@ -104,6 +87,20 @@ get_charset ()
 /* BIDI support (will be moved to lib when it's ready) */
 #ifdef HAVE_LIBFRIBIDI
 
+# ifdef HAVE_FRIBIDI_WCWIDTH
+#  define mu_fribidi_wcwidth fribidi_wcwidth
+# else
+#  if defined(HAVE_WCHAR_H) && defined(HAVE_WCWIDTH)
+#   include <wchar.h>
+#   define mu_fribidi_wcwidth(c) wcwidth((wchar_t)c)
+#  else
+#   undef HAVE_LIBFRIBIDI
+#  endif
+# endif
+#endif
+
+#ifdef HAVE_LIBFRIBIDI
+
 static int fb_charset_num = -1;
 FriBidiChar *logical;
 char *outstring;
@@ -122,14 +119,14 @@ puts_bidi (char *string)
 {
   if (fb_charset_num == -1)
     {
-      fb_charset_num = fribidi_parse_charset (get_charset ());
+      fb_charset_num = fribidi_parse_charset ((char*) get_charset ());
       if (fb_charset_num && frm_debug)
 	mu_error (_("fribidi failed to recognize charset `%s'"),
 		  get_charset ());
     }
   
   if (fb_charset_num == 0)
-    puts (string);
+    mu_printf ("%s\n", string);
   else
     {
       FriBidiStrIndex len;
@@ -170,7 +167,7 @@ puts_bidi (char *string)
 	      if (fb_charset_num != FRIBIDI_CHARSET_CAP_RTL)
 		{
 		  while (wid > 0 && idx < len)
-		    wid -= fribidi_wcwidth (visual[idx++]);
+		    wid -= mu_fribidi_wcwidth (visual[idx++]);
 		}
 	      else
 		{
@@ -188,20 +185,20 @@ puts_bidi (char *string)
 	      new_len = fribidi_unicode_to_charset (fb_charset_num,
 						    visual + st, inlen,
 						    outstring);
-	      printf ("%s", outstring);
+	      mu_printf ("%s", outstring);
 	    }
-	  putchar ('\n');
+	  mu_printf ("\n");
 	}
       else
 	{
 	  /* Print the string as is */
-	  puts (string);
+	  mu_printf ("%s\n", string);
 	}
     }
 }
 #else
 # define alloc_logical(s)
-# define puts_bidi puts
+# define puts_bidi(s) mu_printf ("%s\n", s)
 #endif
 
 
@@ -238,7 +235,7 @@ print_line ()
       curcol = nextstart = 0;
     }
   else
-    putchar ('\n');
+    mu_printf ("\n");
   curfield = 0;
 }
 
@@ -247,9 +244,9 @@ format_field_simple (const char *fmt, ...)
 {
   va_list ap;
   if (curfield++)
-    putchar ('\t');
+    mu_printf ("\t");
   va_start (ap, fmt);
-  vprintf (fmt, ap);
+  mu_stream_vprintf (mu_strout, fmt, ap);
   va_end (ap);
 }
 
@@ -268,7 +265,7 @@ format_field_align (const char *fmt, ...)
 	    {
 	      puts_bidi (linebuf);
 	      linepos = 0;
-	      printf ("%*s", nextstart, "");
+	      mu_printf ("%*s", nextstart, "");
 	    }
 	  else
 	    {
@@ -371,7 +368,7 @@ rfc2047_decode_wrapper (const char *buf, size_t buflen)
   if (rc)
     {
       if (frm_debug)
-	mu_error (_("Cannot decode line `%s': %s"),
+	mu_error (_("cannot decode line `%s': %s"),
 		  buf, mu_strerror (rc));
       return strdup (buf);
     }
@@ -390,41 +387,37 @@ get_personal (mu_header_t hdr, const char *field, char **personal)
   if (status == 0)
     {
       mu_address_t address = NULL;
-      char *s = NULL;
+      const char *s = NULL;
       
       mu_address_create (&address, hfield);
       
-      mu_address_aget_personal (address, 1, &s);
-      mu_address_destroy (&address);
+      mu_address_sget_personal (address, 1, &s);
       if (s == NULL)
 	s = hfield;
-      else
-	free (hfield);
-
       *personal = rfc2047_decode_wrapper (s, strlen (s));
-      free (s);
+      mu_address_destroy (&address);
     }
   return status;
 }
 
-/* Observer action used to perform mailbox scanning. See the comment
-   to frm_scan below.
+/* Observer action is used to perform mailbox scanning. See the comment
+   to frm_scan below. */
 
-   FIXME: The observer action paradygm does not allow for making
-   procedure-data closure, as it should. So, for the time being
-   the following static variables are used instead: */
-
-static frm_select_t select_message;  /* Message selection function */
-static size_t msg_index;             /* Index (1-based) of the current
-					message */
+struct frm_action_closure
+{
+  frm_select_t select_message;  /* Message selection function */
+  size_t msg_index;             /* Index (1-based) of the current
+				   message */
+};
 
 /* Observable action is being called on discovery of each message. */
 /* FIXME: The format of the display is poorly done, please correct.  */
 static int
-action (mu_observer_t o, size_t type)
+action (mu_observer_t o, size_t type, void *data, void *action_data)
 {
   int status;
-
+  struct frm_action_closure *clos = action_data;
+  
   switch (type)
     {
     case MU_EVT_MESSAGE_ADD:
@@ -434,17 +427,17 @@ action (mu_observer_t o, size_t type)
 	mu_header_t hdr = NULL;
 	mu_attribute_t attr = NULL;
 
-	msg_index++;
+	clos->msg_index++;
 	
-	mu_mailbox_get_message (mbox, msg_index, &msg);
+	mu_mailbox_get_message (mbox, clos->msg_index, &msg);
 	mu_message_get_attribute (msg, &attr);
 	mu_message_get_header (msg, &hdr);
 
-	if (!select_message (msg_index, msg))
+	if (!clos->select_message (clos->msg_index, msg))
 	  break;
 
 	if (show_number)
-	  format_field ("%4lu:", (u_long) msg_index);
+	  format_field ("%4lu:", (u_long) clos->msg_index);
 
 	if (show_to)
 	  {
@@ -485,7 +478,7 @@ action (mu_observer_t o, size_t type)
 	      format_field ("");
 
 	    status = mu_header_aget_value_unfold (hdr, MU_HEADER_SUBJECT,
-					       &tmp);
+						  &tmp);
 	    if (status == 0)
 	      {
 		char *s = rfc2047_decode_wrapper (tmp, strlen (tmp));
@@ -513,8 +506,8 @@ frm_abort (mu_mailbox_t *mbox)
   if ((status = mu_mailbox_close (*mbox)) != 0)
     {
       mu_url_t url;
-      
-      mu_error (_("Could not close mailbox `%s': %s"),
+      mu_mailbox_get_url (*mbox, &url);
+      mu_error (_("could not close mailbox `%s': %s"),
 		mu_url_to_string (url), mu_strerror (status));
       exit (3);
     }
@@ -528,7 +521,7 @@ frm_abort (mu_mailbox_t *mbox)
    (mu_message_t). It returns non-zero if that message is to be displayed
    and zero otherwise.
 
-   Upon finishing scanning, the function places the overall number of
+   Upon finishing scanning, the function places total number of
    the messages processed into the memory location pointed to by
    TOTAL */
 void
@@ -542,19 +535,18 @@ frm_scan (char *mailbox_name, frm_select_t fun, size_t *total)
   if (status != 0)
     {
       if (mailbox_name)
-	mu_error (_("Could not create mailbox `%s': %s"),
+	mu_error (_("could not create mailbox `%s': %s"),
 		  mailbox_name,  mu_strerror (status));
       else
-	mu_error (_("Could not create default mailbox: %s"),
+	mu_error (_("could not create default mailbox: %s"),
 		  mu_strerror (status));
       exit (3);
     }
 
   if (frm_debug)
     {
-      mu_debug_t debug;
-      mu_mailbox_get_debug (mbox, &debug);
-      mu_debug_set_level (debug, MU_DEBUG_TRACE|MU_DEBUG_PROT);
+      mu_debug_set_category_level (MU_DEBCAT_MAILBOX, 
+                                   MU_DEBUG_LEVEL_UPTO (MU_DEBUG_PROT));
     }
 
   mu_mailbox_get_url (mbox, &url);
@@ -564,27 +556,27 @@ frm_scan (char *mailbox_name, frm_select_t fun, size_t *total)
     *total = 0;
   else if (status != 0)
     {
-      mu_error (_("Could not open mailbox `%s': %s"),
+      mu_error (_("could not open mailbox `%s': %s"),
 		mu_url_to_string (url), mu_strerror (status));
-      frm_abort (&mbox);
+      mu_mailbox_destroy (&mbox);
+      exit (3);
     }
   else
     {
       mu_observer_t observer;
       mu_observable_t observable;
-
-      select_message = fun;
-      msg_index = 0;
+      struct frm_action_closure closure = { fun, 0 };
       
       mu_observer_create (&observer, mbox);
       mu_observer_set_action (observer, action, mbox);
+      mu_observer_set_action_data (observer, &closure, mbox);
       mu_mailbox_get_observable (mbox, &observable);
       mu_observable_attach (observable, MU_EVT_MESSAGE_ADD, observer);
       
       status = mu_mailbox_scan (mbox, 1, total);
       if (status != 0)
 	{
-	  mu_error (_("Could not scan mailbox `%s': %s."),
+	  mu_error (_("could not scan mailbox `%s': %s"),
 		    mu_url_to_string (url), mu_strerror (status));
 	  frm_abort (&mbox);
 	}
@@ -594,7 +586,7 @@ frm_scan (char *mailbox_name, frm_select_t fun, size_t *total)
       
       if ((status = mu_mailbox_close (mbox)) != 0)
 	{
-	  mu_error (_("Could not close mailbox `%s': %s"),
+	  mu_error (_("could not close mailbox `%s': %s"),
 		    mu_url_to_string (url), mu_strerror (status));
 	  exit (3);
 	}
