@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005, 2006, 2007, 2008, 2010, 2011 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005-2008, 2010-2012 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -83,7 +83,7 @@ _auth_try (void *item, void *data)
 void
 imap4d_auth_capability ()
 {
-  mu_list_do (imap_auth_list, _auth_capa, NULL);
+  mu_list_foreach (imap_auth_list, _auth_capa, NULL);
 }
 
 /*
@@ -112,7 +112,7 @@ imap4d_authenticate (struct imap4d_command *command, imap4d_tokbuf_t tok)
   adata.auth_type = auth_type;
   adata.username = NULL;
 
-  res = mu_list_do (imap_auth_list, _auth_try, &adata);
+  res = mu_list_foreach (imap_auth_list, _auth_try, &adata);
 
   switch (res)
     {
