@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2004, 
-   2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001-2012 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "imap4d.h"
 #ifdef WITH_GSASL
@@ -25,7 +22,7 @@
 #include "tcpwrap.h"
 
 mu_m_server_t server;
-unsigned int idle_timeout;
+unsigned int idle_timeout = 1800;
 int imap4d_transcript;
 
 mu_mailbox_t mbox;              /* Current mailbox */
@@ -54,36 +51,25 @@ int ident_port;
 char *ident_keyfile;
 int ident_encrypt_only;
 
+int test_mode;
+
 const char *program_version = "imap4d (" PACKAGE_STRING ")";
-static char doc[] = N_("GNU imap4d -- the IMAP4D daemon");
+static char doc[] = N_("GNU imap4d -- the IMAP4D daemon.");
 
-#define OPT_LOGIN_DISABLED  256
-#define OPT_TLS_REQUIRED    257
-#define OPT_CREATE_HOME_DIR 258
 #define OPT_PREAUTH         259
 #define OPT_FOREGROUND      260
+#define OPT_TEST_MODE       261
 
 static struct argp_option options[] = {
   { "foreground", OPT_FOREGROUND, 0, 0, N_("remain in foreground"), 0},
   { "inetd",  'i', 0, 0, N_("run in inetd mode"), 0},
   { "daemon", 'd', N_("NUMBER"), OPTION_ARG_OPTIONAL,
     N_("runs in daemon mode with a maximum of NUMBER children"), 0 },
-
-  {"other-namespace", 'O', N_("PATHLIST"), OPTION_HIDDEN,
-   N_("set the `other' namespace"), 0},
-  {"shared-namespace", 'S', N_("PATHLIST"), OPTION_HIDDEN,
-   N_("set the `shared' namespace"), 0},
-  {"login-disabled", OPT_LOGIN_DISABLED, NULL, OPTION_HIDDEN,
-   N_("disable LOGIN command")},
-  {"create-home-dir", OPT_CREATE_HOME_DIR, N_("MODE"),
-   OPTION_ARG_OPTIONAL|OPTION_HIDDEN,
-   N_("create home directory, if it does not exist")},
-  {"preauth", OPT_PREAUTH, NULL, 0,
-   N_("start in preauth mode") },
-#ifdef WITH_TLS
-  {"tls-required", OPT_TLS_REQUIRED, NULL, OPTION_HIDDEN,
-   N_("always require STARTTLS before entering authentication phase")},
-#endif
+  { "test", OPT_TEST_MODE, 0, 0,
+    N_("run in test mode"), 0 },
+  { "preauth", OPT_PREAUTH, NULL, 0,
+    N_("start in preauth mode") },
+  
   {NULL, 0, NULL, 0, NULL, 0}
 };
 
@@ -107,67 +93,47 @@ static const char *imap4d_capa[] = {
   "mailbox",
   "locking",
   "logging",
-  "license",
   NULL
 };
 
-static int imap4d_mainloop (int, FILE *, FILE *);
+static int imap4d_mainloop (int, int, int);
 
 static error_t
 imap4d_parse_opt (int key, char *arg, struct argp_state *state)
 {
-  static struct mu_argp_node_list lst;
+  static mu_list_t lst;
 
   switch (key)
     {
     case 'd':
-      mu_argp_node_list_new (&lst, "mode", "daemon");
+      mu_argp_node_list_new (lst, "mode", "daemon");
       if (arg)
-	mu_argp_node_list_new (&lst, "max-children", arg);
+	mu_argp_node_list_new (lst, "max-children", arg);
       break;
 
     case 'i':
-      mu_argp_node_list_new (&lst, "mode", "inetd");
+      mu_argp_node_list_new (lst, "mode", "inetd");
       break;
 
     case OPT_FOREGROUND:
-      mu_argp_node_list_new (&lst, "foreground", "yes");
+      mu_argp_node_list_new (lst, "foreground", "yes");
       break;
       
-    case 'O':
-      mu_argp_node_list_new (&lst, "other-namespace", arg);
-      break;
-
-    case 'S':
-      mu_argp_node_list_new (&lst, "shared-namespace", arg);
-      break;
-
-    case OPT_LOGIN_DISABLED:
-      mu_argp_node_list_new (&lst, "login-disabled", "yes");
-      break;
-
-    case OPT_CREATE_HOME_DIR:
-      mu_argp_node_list_new (&lst, "create-home-dir", "yes");
-      if (arg)
-	mu_argp_node_list_new (&lst, "home-dir-mode", arg);
-      break;
-	
-#ifdef WITH_TLS
-    case OPT_TLS_REQUIRED:
-      mu_argp_node_list_new (&lst, "tls-required", "yes");
-      break;
-#endif
-
     case OPT_PREAUTH:
       preauth_mode = preauth_stdio;
       break;
+
+    case OPT_TEST_MODE:
+      mu_argp_node_list_new (lst, "mode", "inetd");
+      test_mode = 1;
+      break;
       
     case ARGP_KEY_INIT:
       mu_argp_node_list_init (&lst);
       break;
       
     case ARGP_KEY_FINI:
-      mu_argp_node_list_finish (&lst, NULL, NULL);
+      mu_argp_node_list_finish (lst, NULL, NULL);
       break;
       
     default:
@@ -177,21 +143,19 @@ imap4d_parse_opt (int key, char *arg, struct argp_state *state)
 }
 
 static int
-cb_mode (mu_debug_t debug, void *data, mu_config_value_t *val)
+cb_mode (void *data, mu_config_value_t *val)
 {
   char *p;
-  if (mu_cfg_assert_value_type (val, MU_CFG_STRING, debug))
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
     return 1;
   home_dir_mode = strtoul (val->v.string, &p, 8);
   if (p[0] || (home_dir_mode & ~0777))
-    mu_cfg_format_error (debug, MU_DEBUG_ERROR, 
-                         _("invalid mode specification: %s"),
-			 val->v.string);
+    mu_error (_("invalid mode specification: %s"), val->v.string);
   return 0;
 }
 
 int
-parse_preauth_scheme (mu_debug_t debug, const char *scheme, mu_url_t url)
+parse_preauth_scheme (const char *scheme, mu_url_t url)
 {
   int rc = 0;
   if (strcmp (scheme, "stdio") == 0)
@@ -202,9 +166,7 @@ parse_preauth_scheme (mu_debug_t debug, const char *scheme, mu_url_t url)
       rc = mu_url_aget_path (url, &path);
       if (rc)
 	{
-	  mu_cfg_format_error (debug, MU_DEBUG_ERROR,
-			       _("URL error: cannot get path: %s"),
-			       mu_strerror (rc));
+	  mu_error (_("URL error: cannot get path: %s"), mu_strerror (rc));
 	  return 1;
 	}
       preauth_program = path;
@@ -213,7 +175,7 @@ parse_preauth_scheme (mu_debug_t debug, const char *scheme, mu_url_t url)
   else if (strcmp (scheme, "ident") == 0)
     {
       struct servent *sp;
-      long n;
+      unsigned n;
       if (url && mu_url_get_port (url, &n) == 0)
 	ident_port = (short) n;
       else if ((sp = getservbyname ("auth", "tcp")))
@@ -224,7 +186,7 @@ parse_preauth_scheme (mu_debug_t debug, const char *scheme, mu_url_t url)
     }
   else
     {
-      mu_cfg_format_error (debug, MU_DEBUG_ERROR, _("unknown preauth scheme"));
+      mu_error (_("unknown preauth scheme"));
       rc = 1;
     }
 
@@ -236,14 +198,14 @@ parse_preauth_scheme (mu_debug_t debug, const char *scheme, mu_url_t url)
    preauth stdio
 */
 static int
-cb_preauth (mu_debug_t debug, void *data, mu_config_value_t *val)
+cb_preauth (void *data, mu_config_value_t *val)
 {
-  if (mu_cfg_assert_value_type (val, MU_CFG_STRING, debug))
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
     return 1;
   if (strcmp (val->v.string, "stdio") == 0)
     preauth_mode = preauth_stdio;
   else if (strcmp (val->v.string, "ident") == 0)
-    return parse_preauth_scheme (debug, val->v.string, NULL);
+    return parse_preauth_scheme (val->v.string, NULL);
   else if (val->v.string[0] == '/')
     {
       preauth_program = xstrdup (val->v.string);
@@ -260,24 +222,16 @@ cb_preauth (mu_debug_t debug, void *data, mu_config_value_t *val)
 	  mu_diag_funcall (MU_DIAG_ERROR, "mu_url_create", val->v.string, rc);
 	  return 1;
 	}
-      rc = mu_url_parse (url);
-      if (rc)
-	{
-	  mu_cfg_format_error (debug, MU_DEBUG_ERROR,
-			       "%s: %s", val->v.string, mu_strerror (rc));
-	  return 1;
-	}
 
       rc = mu_url_aget_scheme (url, &scheme);
       if (rc)
 	{
 	  mu_url_destroy (&url);
-	  mu_cfg_format_error (debug, MU_DEBUG_ERROR,
-			       _("URL error: %s"), mu_strerror (rc));
+	  mu_error (_("URL error: %s"), mu_strerror (rc));
 	  return 1;
 	}
 
-      rc = parse_preauth_scheme (debug, scheme, url);
+      rc = parse_preauth_scheme (scheme, url);
       mu_url_destroy (&url);
       free (scheme);
       return rc;
@@ -286,17 +240,30 @@ cb_preauth (mu_debug_t debug, void *data, mu_config_value_t *val)
 }
 
 static int
-cb_mailbox_mode (mu_debug_t debug, void *data, mu_config_value_t *val)
+cb_mailbox_mode (void *data, mu_config_value_t *val)
 {
   const char *p;
-  if (mu_cfg_assert_value_type (val, MU_CFG_STRING, debug))
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
     return 1;
   if (mu_parse_stream_perm_string ((int *)data, val->v.string, &p))
-    mu_cfg_format_error (debug, MU_DEBUG_ERROR,
-			 _("invalid mode string near %s"), p);
+    mu_error (_("invalid mode string near %s"), p);
   return 0;
 }
 
+struct imap4d_srv_config
+{
+  struct mu_srv_config m_cfg;
+  int tls;
+};
+
+static struct mu_cfg_param imap4d_srv_param[] = {
+  { "tls", mu_cfg_bool, NULL, mu_offsetof (struct imap4d_srv_config, tls),
+    NULL,
+    N_("Use TLS encryption for this server")
+  },
+  { NULL }
+};
+
 static struct mu_cfg_param imap4d_cfg_param[] = {
   { "homedir", mu_cfg_string, &modify_homedir, 0, NULL,
     N_("Modify home directory.") },
@@ -339,8 +306,11 @@ static struct mu_cfg_param imap4d_cfg_param[] = {
     N_("Use only encrypted ident responses.") },
   { "id-fields", MU_CFG_LIST_OF(mu_cfg_string), &imap4d_id_list, 0, NULL,
     N_("List of fields to return in response to ID command.") },
+  { "mandatory-locking", mu_cfg_section },
   { ".server", mu_cfg_section, NULL, 0, NULL,
     N_("Server configuration.") },
+  { "transcript", mu_cfg_bool, &imap4d_transcript, 0, NULL,
+    N_("Set global transcript mode.") },
   TCP_WRAPPERS_CONFIG
   { NULL }
 };
@@ -354,21 +324,35 @@ imap4d_session_setup0 ()
 
   if (modify_homedir)
     {
-      int rc;
-      mu_vartab_t vtab;
-      char *expr = mu_tilde_expansion (modify_homedir, "/", real_homedir);
+      char *expr = mu_tilde_expansion (modify_homedir, MU_HIERARCHY_DELIMITER,
+				       real_homedir);
+      struct mu_wordsplit ws;
+      const char *env[5];
 
-      mu_vartab_create (&vtab);
-      mu_vartab_define (vtab, "user", auth_data->name, 0);
-      mu_vartab_define (vtab, "home", real_homedir, 0);
-      rc = mu_vartab_expand (vtab, expr, &imap4d_homedir);
-      mu_vartab_destroy (&vtab);
-      free (expr);
-      if (rc)
+      env[0] = "user";
+      env[1] = auth_data->name;
+      env[2] = "home";
+      env[3] = real_homedir;
+      env[4] = NULL;
+
+      ws.ws_env = env;
+      if (mu_wordsplit (expr, &ws,
+			MU_WRDSF_NOSPLIT | MU_WRDSF_NOCMD |
+			MU_WRDSF_ENV | MU_WRDSF_ENV_KV))
 	{
-	  free (real_homedir);
-	  mu_diag_funcall (MU_DIAG_ERROR, "mu_vartab_expand",
-			   modify_homedir, rc);
+	  mu_error (_("cannot expand line `%s': %s"), expr,
+		    mu_wordsplit_strerror (&ws));
+	  return 1;
+	}
+      else if (ws.ws_wordc == 0)
+	{
+	  mu_error (_("expanding %s yields empty string"), expr);
+	  return 1;
+	}
+      imap4d_homedir = strdup (ws.ws_wordv[0]);
+      if (!imap4d_homedir)
+	{
+	  mu_error ("%s", mu_strerror (errno));
 	  return 1;
 	}
     }
@@ -389,9 +373,14 @@ imap4d_session_setup0 ()
 
   util_chdir (imap4d_homedir);
   namespace_init_session (imap4d_homedir);
+  
   mu_diag_output (MU_DIAG_INFO,
 		  _("user `%s' logged in (source: %s)"), auth_data->name,
 		  auth_data->source);
+
+  if (auth_data->quota)
+    quota_setup ();
+  
   return 0;
 }
 
@@ -410,7 +399,7 @@ imap4d_session_setup (char *username)
 int
 get_client_address (int fd, struct sockaddr_in *pcs)
 {
-  int len = sizeof *pcs;
+  socklen_t len = sizeof *pcs;
 
   if (getpeername (fd, (struct sockaddr *) pcs, &len) < 0)
     {
@@ -419,62 +408,183 @@ get_client_address (int fd, struct sockaddr_in *pcs)
     }
   return 0;
 }
+
+static int
+set_strerr_flt ()
+{
+  mu_stream_t flt, trans[2];
+  int rc;
+  
+  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_SUBSTREAM, MU_IOCTL_OP_GET, trans);
+  if (rc == 0)
+    {
+      char sessidstr[10];
+      char *argv[] = { "inline-comment", NULL, "-S", NULL };
+
+      snprintf (sessidstr, sizeof sessidstr, "%08lx:", mu_session_id);
+      argv[1] = sessidstr;
+      rc = mu_filter_create_args (&flt, trans[0], "inline-comment", 3,
+				  (const char **)argv,
+				  MU_FILTER_ENCODE, MU_STREAM_WRITE);
+      mu_stream_unref (trans[0]);
+      if (rc == 0)
+	{
+	  mu_stream_set_buffer (flt, mu_buffer_line, 0);
+	  trans[0] = flt;
+	  trans[1] = NULL;
+	  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_SUBSTREAM,
+				MU_IOCTL_OP_SET, trans);
+	  mu_stream_unref (trans[0]);
+	  if (rc)
+	    mu_error (_("%s failed: %s"), "MU_IOCTL_SET_STREAM",
+		      mu_stream_strerror (mu_strerr, rc));
+	}
+      else
+	mu_error (_("cannot create log filter stream: %s"), mu_strerror (rc));
+    }
+  else
+    {
+      mu_error (_("%s failed: %s"), "MU_IOCTL_GET_STREAM",
+		mu_stream_strerror (mu_strerr, rc));
+    }
+  return rc;
+}
+
+static void
+clr_strerr_flt ()
+{
+  mu_stream_t flt, trans[2];
+  int rc;
+
+  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_SUBSTREAM, MU_IOCTL_OP_GET, trans);
+  if (rc == 0)
+    {
+      flt = trans[0];
+
+      rc = mu_stream_ioctl (flt, MU_IOCTL_SUBSTREAM, MU_IOCTL_OP_GET, trans);
+      if (rc == 0)
+	{
+	  mu_stream_unref (trans[0]);
+	  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_SUBSTREAM,
+				MU_IOCTL_OP_SET, trans);
+	  if (rc == 0)
+	    mu_stream_unref (flt);
+	}
+    }
+}
+
+void
+imap4d_child_signal_setup (RETSIGTYPE (*handler) (int signo))
+{
+  static int sigtab[] = { SIGPIPE, SIGABRT, SIGINT, SIGQUIT,
+			  SIGTERM, SIGHUP, SIGALRM };
+  mu_set_signals (handler, sigtab, MU_ARRAY_SIZE (sigtab));
+}
 
 static int
-imap4d_mainloop (int fd, FILE *infile, FILE *outfile)
+imap4d_mainloop (int ifd, int ofd, int tls)
 {
   imap4d_tokbuf_t tokp;
   char *text;
-  int debug_mode = isatty (fd);
-  static int sigtab[] = { SIGILL, SIGBUS, SIGFPE, SIGSEGV, SIGSTOP, SIGPIPE,
-			  SIGABRT, SIGINT, SIGQUIT, SIGTERM, SIGHUP, SIGALRM };
+  int signo;
 
-  /* Reset signals */
-  mu_set_signals (imap4d_child_signal, sigtab, MU_ARRAY_SIZE (sigtab));
-  
-  util_setio (infile, outfile);
+  if (!test_mode)
+    test_mode = isatty (ifd);
 
-  if (imap4d_preauth_setup (fd) == 0)
+  if ((signo = setjmp (child_jmp)))
     {
-      if (debug_mode)
+      mu_diag_output (MU_DIAG_CRIT, _("got signal `%s'"), strsignal (signo));
+      switch (signo)
 	{
-	  mu_diag_output (MU_DIAG_INFO, _("started in debugging mode"));
-	  text = "IMAP4rev1 Debugging mode";
+	case SIGTERM:
+	case SIGHUP:
+	  signo = ERR_TERMINATE;
+	  break;
+      
+	case SIGALRM:
+	  signo = ERR_TIMEOUT;
+	  break;
+      
+	case SIGPIPE:
+	  signo = ERR_NO_OFILE;
+	  break;
+      
+	default:
+	  signo = ERR_SIGNAL;
+	}
+      imap4d_bye (signo);
+    }
+  else
+    {
+      /* Restore default handling for these signals: */
+      static int defsigtab[] = { SIGILL, SIGBUS, SIGFPE, SIGSEGV, SIGSTOP };
+      mu_set_signals (SIG_DFL, defsigtab, MU_ARRAY_SIZE (defsigtab));
+      /* Set child-specific signal handlers */
+      imap4d_child_signal_setup (imap4d_child_signal);
+    }
+  
+  io_setio (ifd, ofd, tls);
+
+  if (imap4d_preauth_setup (ifd) == 0)
+    {
+      if (test_mode)
+	{
+	  mu_diag_output (MU_DIAG_INFO, _("started in test mode"));
+	  text = "IMAP4rev1 Test mode";
 	}
       else
 	text = "IMAP4rev1";
     }
   else
     {
-      util_flush_output ();
+      io_flush ();
       return 0;
     }
 
-  /* Greetings.  */
-  util_out ((state == STATE_AUTH) ? RESP_PREAUTH : RESP_OK, "%s", text);
-  util_flush_output ();
+  /* Greetings. */
+  io_untagged_response ((state == STATE_AUTH) ? 
+                        RESP_PREAUTH : RESP_OK, "%s", text);
+  io_flush ();
 
+  set_xscript_level ((state == STATE_AUTH) ?
+                      MU_XSCRIPT_NORMAL : MU_XSCRIPT_SECURE);
+  
   tokp = imap4d_tokbuf_init ();
   while (1)
     {
+      if (idle_timeout && io_wait_input (idle_timeout) != 1)
+	imap4d_bye (ERR_TIMEOUT);
       imap4d_readline (tokp);
       /* check for updates */
       imap4d_sync ();
       util_do_command (tokp);
       imap4d_sync ();
-      util_flush_output ();
+      io_flush ();
     }
 
   return 0;
 }
 
 int
-imap4d_connection (int fd, struct sockaddr *sa, int salen, void *data,
-		   mu_ip_server_t srv, time_t timeout, int transcript)
+imap4d_connection (int fd, struct sockaddr *sa, int salen,
+		   struct mu_srv_config *pconf, void *data)
 {
-  idle_timeout = timeout;
-  imap4d_transcript = transcript;
-  imap4d_mainloop (fd, fdopen (fd, "r"), fdopen (fd, "w"));
+  struct imap4d_srv_config *cfg = (struct imap4d_srv_config *) pconf;
+  int rc;
+  
+  idle_timeout = pconf->timeout;
+  imap4d_transcript = pconf->transcript;
+
+  if (mu_log_session_id)
+    rc = set_strerr_flt ();
+  else
+    rc = 1;
+  
+  imap4d_mainloop (fd, fd, cfg->tls);
+
+  if (rc == 0)
+    clr_strerr_flt ();
+  
   return 0;
 }
 
@@ -508,6 +618,16 @@ imap4d_check_home_dir (const char *dir, uid_t uid, gid_t gid)
   return 0;
 }
 
+
+jmp_buf master_jmp;
+
+RETSIGTYPE
+imap4d_master_signal (int signo)
+{
+  longjmp (master_jmp, signo);
+}
+
+
 int
 main (int argc, char **argv)
 {
@@ -536,12 +656,14 @@ main (int argc, char **argv)
   mu_gocs_register ("gsasl", mu_gsasl_module_init);
 #endif
   mu_tcpwrapper_cfg_init ();
+  manlock_cfg_init ();
   mu_acl_cfg_init ();
-  mu_m_server_cfg_init ();
+  mu_m_server_cfg_init (imap4d_srv_param);
   
-  mu_argp_init (program_version, NULL);
+  mu_argp_init (NULL, NULL);
 
   mu_m_server_create (&server, program_version);
+  mu_m_server_set_config_size (server, sizeof (struct imap4d_srv_config));
   mu_m_server_set_conn (server, imap4d_connection);
   mu_m_server_set_prefork (server, mu_tcp_wrapper_prefork);
   mu_m_server_set_mode (server, MODE_INTERACTIVE);
@@ -551,6 +673,8 @@ main (int argc, char **argv)
   mu_m_server_set_timeout (server, 1800);  /* RFC2060: 30 minutes. */
   mu_m_server_set_strexit (server, mu_strexit);
   
+  mu_log_syslog = 1;
+
   if (mu_app_init (&argp, imap4d_capa, imap4d_cfg_param, 
 		   argc, argv, 0, NULL, server))
     exit (EX_CONFIG); /* FIXME: No way to discern from EX_USAGE? */
@@ -563,9 +687,12 @@ main (int argc, char **argv)
 #endif
 
   namespace_init ();
-  
-  auth_gssapi_init ();
-  auth_gsasl_init ();
+
+  if (mu_gsasl_enabled ())
+    {
+      auth_gssapi_init ();
+      auth_gsasl_init ();
+    }
 
 #ifdef USE_LIBPAM
   if (!mu_pam_service)
@@ -600,21 +727,31 @@ main (int argc, char **argv)
     }
 
   /* Set the signal handlers.  */
+  if ((status = setjmp (master_jmp)))
+    {
+      int code;
+      mu_diag_output (MU_DIAG_CRIT, _("MASTER: exiting on signal (%s)"),
+		      strsignal (status));
+      switch (status)
+	{
+	case SIGTERM:
+	case SIGHUP:
+	case SIGQUIT:
+	case SIGINT:
+	  code = EX_OK;
+	  break;
+	  
+	default:
+	  code = EX_SOFTWARE;
+	  break;
+	}
+      
+      exit (code); 
+    }
   mu_set_signals (imap4d_master_signal, sigtab, MU_ARRAY_SIZE (sigtab));
 
-  /* Set up for syslog.  */
-  openlog (MU_LOG_TAG (), LOG_PID, mu_log_facility);
-
-  /* Redirect any stdout error from the library to syslog, they
-     should not go to the client.  */
-  {
-    mu_debug_t debug;
-
-    mu_diag_get_debug (&debug);
-    mu_debug_set_print (debug, mu_diag_syslog_printer, NULL);
-
-    mu_debug_default_printer = mu_debug_syslog_printer;
-  }
+  mu_stdstream_strerr_setup (mu_log_syslog ?
+			     MU_STRERR_SYSLOG : MU_STRERR_STDERR);
 
   umask (S_IROTH | S_IWOTH | S_IXOTH);	/* 007 */
 
@@ -635,7 +772,7 @@ main (int argc, char **argv)
     {
       /* Make sure we are in the root directory.  */
       chdir ("/");
-      status = imap4d_mainloop (fileno (stdin), stdin, stdout);
+      status = imap4d_mainloop (MU_STDIN_FD, MU_STDOUT_FD, 0);
     }
 
   if (status)
