@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
-   2009, 2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001-2012 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -105,6 +104,9 @@
 #include <mailutils/cstr.h>
 #include <mailutils/io.h>
 #include <mailutils/prog.h>
+#include <mailutils/imapio.h>
+#include <mailutils/imaputil.h>
+#include <mailutils/msgset.h>
 
 #include <mu_umaxtostr.h>
 #include <muaux.h>
@@ -204,6 +206,9 @@ extern int imap4d_argc;
 extern char **imap4d_argv;
 extern jmp_buf child_jmp;
 
+extern int test_mode;
+extern int silent_expunge;
+
 /* Input functions */
 extern mu_stream_t iostream;
 extern int  io_untagged_response (int, const char *, ...) MU_PRINTFLIKE(2,3);
@@ -345,10 +350,9 @@ extern mu_list_t namespace[NS_MAX];
   
 extern int namespace_init_session (char *path);
 extern void namespace_init (void);
-extern char *namespace_getfullpath (const char *name, const char *delim,
-				    int *pns);
+extern char *namespace_getfullpath (const char *name, int *pns);
 extern char *namespace_checkfullpath (const char *name, const char *pattern,
-				       const char *delim, int *pns);
+				      int *pns);
 int imap4d_session_setup (char *username);
 int imap4d_session_setup0 (void);
 void imap4d_child_signal_setup (RETSIGTYPE (*handler) (int signo));
@@ -363,9 +367,7 @@ extern void imap4d_capability_init (void);
 extern int  util_start (char *);
 extern int  util_getstate (void);
 extern int  util_do_command (imap4d_tokbuf_t);
-extern char *util_tilde_expansion (const char *, const char *);
-extern char *util_getfullpath (const char *, const char *);
-extern int  util_msgset (char *, size_t **, int *, int);
+extern char *util_getfullpath (const char *);
 extern struct imap4d_command *util_getcommand (char *, 
                                                struct imap4d_command []);
 
@@ -385,12 +387,7 @@ extern int util_parse_ctime_date (const char *date, time_t *timep,
 extern char *util_strcasestr (const char *haystack, const char *needle);
 extern char *util_localname (void);
 
-extern int util_wcard_match (const char *string, const char *pattern,
-			     const char *delim);
-int util_format_attribute_flags (mu_stream_t str, int flags);
 void util_print_flags (mu_attribute_t attr);
-int util_attribute_to_type (const char *item, int *type);
-int util_type_to_attribute (int type, char **attr_str);
 int util_attribute_matches_flag (mu_attribute_t attr, const char *item);
 int util_uidvalidity (mu_mailbox_t smbox, unsigned long *uidvp);
 
