@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005, 2007-2012 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "imap4d.h"
 
@@ -22,12 +21,12 @@ typedef int (*nsfp_t) (void *closure, int ns, char *path, int delim);
 
 mu_list_t namespace[NS_MAX];
 
-static char *
-printable_pathname (char *str)
+static const char *
+printable_pathname (const char *str)
 {
-  if (strncmp (str, homedir, strlen (homedir)) == 0)
+  if (strncmp (str, imap4d_homedir, strlen (imap4d_homedir)) == 0)
     {
-      str += strlen (homedir);
+      str += strlen (imap4d_homedir);
       if (str[0] == '/')
 	str++;
     }
@@ -38,11 +37,11 @@ static int
 print_namespace_fun (void *item, void *data)
 {
   int *pcount = data;
-  char *dir = printable_pathname (item);
+  const char *dir = printable_pathname (item);
   char *suf = (dir[0] && dir[strlen (dir) - 1] != '/') ? "/" : "";
   if ((*pcount)++)
-    util_send (" ");
-  util_send ("(\"%s%s\" \"/\")", dir, suf);
+    io_sendf (" ");
+  io_sendf ("(\"%s%s\" \"/\")", dir, suf);
   return 0;
 }
 
@@ -51,14 +50,14 @@ print_namespace (int nsid)
 {
   mu_list_t list = namespace[nsid];
   if (!list)
-    util_send ("NIL");
+    io_sendf ("NIL");
   else
     {
       int count;
       count = 0;
-      util_send ("(");
-      mu_list_do (list, print_namespace_fun, &count);
-      util_send (")");
+      io_sendf ("(");
+      mu_list_foreach (list, print_namespace_fun, &count);
+      io_sendf (")");
     }
 }
 
@@ -85,7 +84,7 @@ namespace_enumerate (int id, nsfp_t f, void *closure)
   nsc.fun = f;
   nsc.closure = closure;
   return namespace[id] == 0 ? 0 :
-          mu_list_do (namespace[id], _enum_fun, &nsc);
+          mu_list_foreach (namespace[id], _enum_fun, &nsc);
 }
 
 static int
@@ -116,18 +115,18 @@ int
 imap4d_namespace (struct imap4d_command *command, imap4d_tokbuf_t tok)
 {
   if (imap4d_tokbuf_argc (tok) != 2)
-    return util_finish (command, RESP_BAD, "Invalid arguments");
+    return io_completion_response (command, RESP_BAD, "Invalid arguments");
 
-  util_send ("* NAMESPACE ");
+  io_sendf ("* NAMESPACE ");
 
   print_namespace (NS_PRIVATE);
-  util_send (" ");
+  io_sendf (" ");
   print_namespace (NS_OTHER);
-  util_send (" ");
+  io_sendf (" ");
   print_namespace (NS_SHARED);
-  util_send ("\r\n");
+  io_sendf ("\n");
 
-  return util_finish (command, RESP_OK, "Completed");
+  return io_completion_response (command, RESP_OK, "Completed");
 }
 
 
@@ -167,7 +166,7 @@ risky_pattern (const char *pattern, int delim)
 
 char *
 namespace_checkfullpath (const char *name, const char *pattern, 
-			 const char *delim, int *nspace)
+			 int *nspace)
 {
   struct namespace_info info;
   const char *p;
@@ -186,7 +185,7 @@ namespace_checkfullpath (const char *name, const char *pattern,
       name = p + 1;
     }
 
-  path = util_getfullpath (name, delim);
+  path = util_getfullpath (name);
   if (!path)
     {
       free (scheme);
@@ -227,26 +226,27 @@ namespace_checkfullpath (const char *name, const char *pattern,
 }
 
 char *
-namespace_getfullpath (const char *name, const char *delim, int *nspace)
+namespace_getfullpath (const char *name, int *nspace)
 {
   char *ret;
-  if (strcasecmp (name, "INBOX") == 0 && auth_data->change_uid)
+  if (mu_c_strcasecmp (name, "INBOX") == 0 && auth_data->change_uid)
     {
       ret = strdup (auth_data->mailbox);
       if (nspace)
 	*nspace = NS_PRIVATE;
     }
   else
-    ret = namespace_checkfullpath (name, NULL, delim, nspace);
+    ret = namespace_checkfullpath (name, NULL, nspace);
   return ret;
 }
 
 int
 namespace_init_session (char *path)
 {
-  mu_list_create (&namespace[NS_PRIVATE]);
-  mu_list_append (namespace[NS_PRIVATE],
-		  mu_strdup (mu_normalize_path (path)));
+  if (!namespace[NS_PRIVATE])
+    mu_list_create (&namespace[NS_PRIVATE]);
+  mu_list_prepend (namespace[NS_PRIVATE],
+		   mu_strdup (mu_normalize_path (path)));
   return 0;
 }
 
@@ -259,12 +259,6 @@ normalize_fun (void *item, void *data)
 			 mu_strdup (mu_normalize_path (name)));
 }
 
-static void
-free_item (void *item)
-{
-  free (item);
-}
-  
 void
 namespace_init ()
 {
@@ -276,9 +270,9 @@ namespace_init ()
 	{
 	  mu_list_t list;
 	  mu_list_create (&list);
-	  mu_list_set_destroy_item (list, free_item);
-	  mu_list_do (namespace[i], normalize_fun, list);
-	  mu_list_set_destroy_item (namespace[i], free_item);
+	  mu_list_set_destroy_item (list, mu_list_free_item);
+	  mu_list_foreach (namespace[i], normalize_fun, list);
+	  mu_list_set_destroy_item (namespace[i], mu_list_free_item);
 	  mu_list_destroy (&namespace[i]);
 	  namespace[i] = list;
 	}
