@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2004, 
-   2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001-2012 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Preauth support for imap4d */
 
@@ -62,22 +59,6 @@ ident_extract_username (char *reply)
   return p;
 }
 
-static int
-trimcrlf (char *buf)
-{
-  int len = strlen (buf);
-  if (len == 0)
-    return 0;
-  if (buf[len-1] == '\n')
-    {
-      len--;
-      if (buf[len-1] == '\r')
-	len--;
-      buf[len] = 0;
-    }
-  return len;
-}
-
 static int
 is_des_p (const char *name)
 {
@@ -141,7 +122,7 @@ des_cbc_cksum (gl_des_ctx *ctx, unsigned char *buf, size_t bufsize,
 	  }
 	  bufsize = 0;
 	}
-      gl_des_ecb_crypt (ctx, key, key, 0);
+      gl_des_ecb_crypt (ctx, (char*) key, (char*) key, 0);
     }
 }
 
@@ -194,8 +175,8 @@ des_string_to_key (char *buf, size_t bufsize, unsigned char key[8])
     }
 
   des_fixup_key_parity (key);
-  gl_des_setkey (&context, key);
-  des_cbc_cksum (&context, buf, bufsize, key, key);
+  gl_des_setkey (&context, (char*) key);
+  des_cbc_cksum (&context, (unsigned char*) buf, bufsize, key, key);
   memset (&context, 0, sizeof context);
   des_fixup_key_parity (key);
 }
@@ -210,14 +191,11 @@ decode64_buf (const char *name, unsigned char **pbuf, size_t *psize)
   
   name++;
   namelen = strlen (name) - 1;
-  mu_memory_stream_create (&str, NULL, MU_STREAM_NO_CHECK);
-  mu_filter_create (&flt, str, "base64", MU_FILTER_DECODE,
-		    MU_STREAM_READ | MU_STREAM_NO_CHECK);
-  mu_stream_open (str);
-  mu_stream_sequential_write (str, name, namelen);
-  mu_stream_read (flt, buf, sizeof buf, 0, &size);
-  mu_stream_destroy (&flt, NULL);
-  mu_stream_destroy (&str, NULL);
+  mu_static_memory_stream_create (&str, name, namelen);
+  mu_filter_create (&flt, str, "base64", MU_FILTER_DECODE, MU_STREAM_READ);
+  mu_stream_unref (str);
+  mu_stream_read (flt, buf, sizeof buf, &size);
+  mu_stream_destroy (&flt);
   *pbuf = malloc (size);
   if (!*pbuf)
     return 1;
@@ -281,7 +259,7 @@ ident_decrypt (const char *file, const char *name)
       gl_des_ctx ctx;
       
       des_string_to_key (keybuf, sizeof (keybuf), key);
-      gl_des_setkey (&ctx, key);
+      gl_des_setkey (&ctx, (char*) key);
       
       memcpy (id.chars, buf, size);
       
@@ -350,8 +328,7 @@ do_preauth_ident (struct sockaddr *clt_sa, struct sockaddr *srv_sa)
   
   memcpy (hostaddr, p, 15);
   hostaddr[15] = 0;
-  rc = mu_tcp_stream_create (&stream, hostaddr, ident_port, 
-			     MU_STREAM_RDWR | MU_STREAM_NO_CHECK);
+  rc = mu_tcp_stream_create (&stream, hostaddr, ident_port, MU_STREAM_RDWR);
   if (rc)
     {
       mu_diag_output (MU_DIAG_INFO, _("cannot create TCP stream: %s"),
@@ -359,22 +336,14 @@ do_preauth_ident (struct sockaddr *clt_sa, struct sockaddr *srv_sa)
       return NULL;
     }
 
-  rc = mu_stream_open (stream);
-  if (rc)
-    {
-      mu_diag_output (MU_DIAG_INFO, _("cannot open TCP stream to %s:%d: %s"),
-		      hostaddr, ident_port, mu_strerror (rc));
-      return NULL;
-    }
-
-  mu_stream_sequential_printf (stream, "%u , %u\r\n",
-			       ntohs (clt_addr->sin_port),
-			       ntohs (srv_addr->sin_port));
+  mu_stream_printf (stream, "%u , %u\n",
+		    ntohs (clt_addr->sin_port),
+		    ntohs (srv_addr->sin_port));
   mu_stream_shutdown (stream, MU_STREAM_WRITE);
 
-  rc = mu_stream_sequential_getline (stream, &buf, &size, NULL);
+  rc = mu_stream_getline (stream, &buf, &size, NULL);
   mu_stream_close (stream);
-  mu_stream_destroy (&stream, NULL);
+  mu_stream_destroy (&stream);
   if (rc)
     {
       mu_diag_output (MU_DIAG_INFO, _("cannot read answer from %s:%d: %s"),
@@ -382,8 +351,7 @@ do_preauth_ident (struct sockaddr *clt_sa, struct sockaddr *srv_sa)
       return NULL;
     }
   mu_diag_output (MU_DIAG_INFO, "Got %s", buf);
-  trimcrlf (buf);
-  name = ident_extract_username (buf);
+  name = ident_extract_username (mu_str_stripws (buf));
   if (!name)
     mu_diag_output (MU_DIAG_INFO,
 		    _("malformed IDENT response: `%s', from %s:%d"),
@@ -418,48 +386,83 @@ do_preauth_ident (struct sockaddr *clt_sa, struct sockaddr *srv_sa)
 }
 
 
+#define SEQ(s, n, l) \
+  (((l) == (sizeof(s) - 1)) && memcmp (s, n, l) == 0)
+
+struct preauth_closure
+{
+  struct sockaddr_in *s_clt, *s_srv;
+};
+
+static const char *
+preauth_getvar (const char *name, size_t nlen, void *data)
+{
+  struct preauth_closure *clos = data;
+  
+  if (clos->s_clt && clos->s_clt->sin_family == AF_INET)
+    {
+      if (SEQ ("client_address", name, nlen))
+        return inet_ntoa (clos->s_clt->sin_addr);
+      if (SEQ ("client_prot", name, nlen))
+        return mu_umaxtostr (0, ntohs (clos->s_clt->sin_port));
+    }
+  if (clos->s_srv && clos->s_srv->sin_family == AF_INET)
+    {
+      if (SEQ ("server_address", name, nlen))
+        return inet_ntoa (clos->s_srv->sin_addr);
+      if (SEQ ("server_port", name, nlen))
+        return mu_umaxtostr (0, ntohs (clos->s_srv->sin_port));
+    }
+  return NULL;
+}
+  
 /* External (program) preauth */
 static char *
 do_preauth_program (struct sockaddr *pcs, struct sockaddr *sa)
 {
   int rc;
-  mu_vartab_t vtab;
-  char *cmd;
-  FILE *fp;
+  mu_stream_t str;
   char *buf = NULL;
-  size_t size = 0;
-  
-  mu_vartab_create (&vtab);
-  if (pcs && pcs->sa_family == AF_INET)
-    {
-      struct sockaddr_in *s_in = (struct sockaddr_in *)pcs;
-      mu_vartab_define (vtab, "client_address", inet_ntoa (s_in->sin_addr), 0);
-      mu_vartab_define (vtab, "client_port",
-			mu_umaxtostr (0, ntohs (s_in->sin_port)), 0);
-    }
-  if (sa && sa->sa_family == AF_INET)
-    {
-      struct sockaddr_in *s_in = (struct sockaddr_in *) sa;
-      mu_vartab_define (vtab, "server_address", inet_ntoa (s_in->sin_addr), 0);
-      mu_vartab_define (vtab, "server_port",
-			mu_umaxtostr (0, ntohs (s_in->sin_port)), 0);
-    }
-  rc = mu_vartab_expand (vtab, preauth_program, &cmd);
-  mu_vartab_destroy (&vtab);
-  if (rc)
-    return NULL;
+  size_t size = 0, n;
+  struct mu_wordsplit ws;
+  struct preauth_closure clos;
 
-  fp = popen (cmd, "r");
-  free (cmd);
-  rc = getline (&buf, &size, fp);
-  pclose (fp);
-  if (rc > 0)
+  clos.s_clt = (struct sockaddr_in *) pcs;
+  clos.s_srv = (struct sockaddr_in *) sa;
+
+  ws.ws_getvar = preauth_getvar;
+  ws.ws_closure = &clos;
+  if (mu_wordsplit (preauth_program, &ws,
+		    MU_WRDSF_NOSPLIT | MU_WRDSF_NOCMD |
+		    MU_WRDSF_GETVAR | MU_WRDSF_CLOSURE))
     {
-      if (trimcrlf (buf) == 0)
-	{
-	  free (buf);
-	  return NULL;
-	}
+      mu_error (_("cannot expand line `%s': %s"), preauth_program,
+		mu_wordsplit_strerror (&ws));
+      return NULL;
+    }
+  else if (ws.ws_wordc == 0)
+    {
+      mu_wordsplit_free (&ws);
+      mu_error (_("`%s' expands to an empty line"), preauth_program);
+      return NULL;
+    }
+
+  rc = mu_command_stream_create (&str, ws.ws_wordv[0], MU_STREAM_READ);
+  mu_wordsplit_free (&ws);
+  if (rc)
+    {
+      mu_error (_("cannot open input pipe from %s"), preauth_program);
+      return NULL;
+    }
+  rc = mu_stream_getline (str, &buf, &size, &n);
+  mu_stream_destroy (&str);
+  if (rc)
+    {
+      mu_error (_("read from `%s' failed"), preauth_program);
+    }
+  else
+    {
+      mu_rtrim_class (buf, MU_CTYPE_ENDLN);
       return buf;
     }
   return NULL;
@@ -469,9 +472,9 @@ int
 imap4d_preauth_setup (int fd)
 {
   struct sockaddr clt_sa, *pclt_sa; 
-  int clt_len = sizeof clt_sa;
+  socklen_t clt_len = sizeof clt_sa;
   struct sockaddr srv_sa, *psrv_sa;
-  int srv_len = sizeof srv_sa;
+  socklen_t srv_len = sizeof srv_sa;
   char *username = NULL;
 
   if (getsockname (fd, &srv_sa, &srv_len) == -1)
@@ -481,8 +484,13 @@ imap4d_preauth_setup (int fd)
     }
   else
     psrv_sa = &srv_sa;
-  
-  if (getpeername (fd, (struct sockaddr *) &clt_sa, &clt_len) == -1)
+
+  if (test_mode)
+    {
+      pclt_sa = NULL;
+      clt_len = 0;
+    }
+  else if (getpeername (fd, (struct sockaddr *) &clt_sa, &clt_len) == -1)
     {
       mu_diag_output (MU_DIAG_ERROR,
 		      _("cannot obtain IP address of client: %s"),
