@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005, 2006, 2007, 2010, 2011 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2001, 2005-2007, 2010-2012 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -43,6 +43,8 @@ extern int  mu_mailbox_create_from_record (mu_mailbox_t *pmbox,
 					   mu_record_t record,
 					   const char *name);
 extern int  mu_mailbox_create_default  (mu_mailbox_t *, const char *);
+extern int mu_mailbox_create_at (mu_mailbox_t *pmbox, mu_folder_t folder,
+				 const char *name);
 
 extern void mu_mailbox_destroy         (mu_mailbox_t *);
 
