@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005, 2007, 2009, 2010, 2011 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2001, 2005, 2007, 2009-2012 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -713,8 +713,8 @@ addr_field_by_mask (mu_address_t addr, int mask)
 {
   switch (mask)						
     {
-    case MU_ADDR_HINT_ADDR:
-      return addr->addr;
+    case MU_ADDR_HINT_PRINTABLE:
+      return addr->printable;
 	  
     case MU_ADDR_HINT_COMMENTS:				
       return addr->comments;					
@@ -758,7 +758,7 @@ addr_free_fields (mu_address_t a, int memflag)
 {
   char *p;
   
-  if ((p = addr_field_by_mask (a, memflag & MU_ADDR_HINT_ADDR)))
+  if ((p = addr_field_by_mask (a, memflag & MU_ADDR_HINT_PRINTABLE)))
     free (p);
   if ((p = addr_field_by_mask (a, memflag & MU_ADDR_HINT_COMMENTS)))
     free (p);
