@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
-   Free Software Foundation, Inc.
+   Copyright (C) 2002-2012 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -13,9 +12,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see 
+   <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -48,10 +46,10 @@
 #include <mailutils/error.h>
 #include <mailutils/errno.h>
 #include <mailutils/nls.h>
-#include <mailutils/mutil.h>
+#include <mailutils/util.h>
 #include <mailutils/sql.h>
-#include <mailutils/vartab.h>
 #include <mailutils/cstr.h>
+#include <mailutils/wordsplit.h>
 #include "sql.h"
 
 #ifdef USE_SQL
@@ -89,23 +87,36 @@ sql_escape_string (const char *ustr)
 char *
 mu_sql_expand_query (const char *query, const char *ustr)
 {
-  int rc;
   char *res;
   char *esc_ustr;
-  mu_vartab_t vtab;
-  
+  struct mu_wordsplit ws;
+  const char *env[2 + 1];
+
   if (!query)
     return NULL;
 
   esc_ustr = sql_escape_string (ustr);
-  mu_vartab_create (&vtab);
-  mu_vartab_define (vtab, "user", ustr, 1);
-  mu_vartab_define (vtab, "u", ustr, 1);
-  rc = mu_vartab_expand (vtab, query, &res);
-  if (rc)
-    res = NULL;
-  mu_vartab_destroy (&vtab);
+  env[0] = "user";
+  env[1] = (char*) ustr;
+  env[2] = NULL;
 
+  ws.ws_env = env;
+  if (mu_wordsplit (query, &ws,
+		    MU_WRDSF_NOSPLIT | MU_WRDSF_NOCMD |
+		    MU_WRDSF_ENV | MU_WRDSF_ENV_KV))
+    {
+      mu_error (_("cannot expand line `%s': %s"), query,
+		mu_wordsplit_strerror (&ws));
+      return NULL;
+    }
+  else if (ws.ws_wordc == 0)
+    {
+      mu_error (_("expanding %s yields empty string"), query);
+      mu_wordsplit_free (&ws);
+      return NULL;
+    }
+  res = strdup (ws.ws_wordv[0]);
+  mu_wordsplit_free (&ws);
   free (esc_ustr);
   return res;
 }
@@ -380,7 +391,17 @@ mu_auth_sql_by_name (struct mu_auth_data **return_data,
       return MU_ERR_FAILURE;
     }
 
-  mu_sql_num_tuples (conn, &n);
+  status = mu_sql_num_tuples (conn, &n);
+  if (status)
+    {
+      mu_error (_("cannot get number of tuples: %s"),
+                (status == MU_ERR_SQL) ?  mu_sql_strerror (conn) :
+                                          mu_strerror (status));
+      mu_sql_release_result (conn);
+      mu_sql_connection_destroy (&conn);
+      return MU_ERR_FAILURE;
+    }
+  
   if (n == 0)
     rc = MU_ERR_AUTH_FAILURE;
   else
@@ -464,7 +485,16 @@ mu_auth_sql_by_uid (struct mu_auth_data **return_data,
       return MU_ERR_FAILURE;
     }
 
-  mu_sql_num_tuples (conn, &n);
+  status = mu_sql_num_tuples (conn, &n);
+  if (status)
+    {
+      mu_error (_("cannot get number of tuples: %s"),
+                (status == MU_ERR_SQL) ?  mu_sql_strerror (conn) :
+                                          mu_strerror (status));
+      mu_sql_release_result (conn);
+      mu_sql_connection_destroy (&conn);
+      return MU_ERR_FAILURE;
+    }
 
   if (n == 0)
     rc = MU_ERR_AUTH_FAILURE;
@@ -485,7 +515,8 @@ mu_sql_getpass (const char *username, char **passwd)
   char *query_str;
   int status;
   char *sql_pass;
-  
+  size_t nt;
+
   query_str = mu_sql_expand_query (mu_sql_module_config.getpass_query, username);
 
   if (!query_str)
@@ -528,7 +559,7 @@ mu_sql_getpass (const char *username, char **passwd)
       mu_sql_connection_destroy (&conn);
       return MU_ERR_FAILURE;
     }
-
+  
   status = mu_sql_store_result (conn);
 
   if (status)
@@ -540,6 +571,23 @@ mu_sql_getpass (const char *username, char **passwd)
       return MU_ERR_FAILURE;
     }
 
+  status = mu_sql_num_tuples (conn, &nt);
+  if (status)
+    {
+      mu_error (_("cannot get number of tuples: %s"),
+                (status == MU_ERR_SQL) ?  mu_sql_strerror (conn) :
+                                          mu_strerror (status));
+      mu_sql_release_result (conn);
+      mu_sql_connection_destroy (&conn);
+      return MU_ERR_FAILURE;
+    }
+  if (nt == 0)
+    {
+      mu_sql_release_result (conn);
+      mu_sql_connection_destroy (&conn);
+      return MU_ERR_FAILURE;
+    }
+
   status = mu_sql_get_column (conn, 0, 0, &sql_pass);
   if (status)
     {
@@ -551,6 +599,14 @@ mu_sql_getpass (const char *username, char **passwd)
       return MU_ERR_FAILURE;
     }
 
+  if (!sql_pass)
+    {
+      mu_error (_("SQL returned NULL password"));
+      mu_sql_release_result (conn);
+      mu_sql_connection_destroy (&conn);
+      return MU_ERR_FAILURE;
+    }
+
   *passwd = strdup (sql_pass);
 
   mu_sql_disconnect (conn);
