@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2007, 2009, 2010,
-   2011 Free Software Foundation, Inc.
+   Copyright (C) 1999-2003, 2005, 2007, 2009-2012 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -346,7 +346,7 @@ get_content_encoding (mu_header_t hdr, char **value)
     {
       if (encoding)
 	free (encoding);
-      encoding = strdup ("7bit"); /* Default.  */
+      encoding = xstrdup ("7bit"); /* Default.  */
     }
   *value = encoding;
   return 0;
