@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005, 2007, 2009-2012 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -30,9 +29,11 @@ size0 (msgset_t *mspec, mu_message_t msg, void *data)
   mu_message_size (msg, &size);
   mu_message_lines (msg, &lines);
   
-  fprintf (ofile, "%c%2d %3d/%-5d\n",
-	   is_current_message (mspec->msg_part[0]) ? '>' : ' ',
-	   mspec->msg_part[0], lines, size);
+  mu_printf ("%c%2lu %3lu/%-5lu\n",
+		    is_current_message (mspec->msg_part[0]) ? '>' : ' ',
+		    (unsigned long) mspec->msg_part[0],
+		    (unsigned long) lines,
+		    (unsigned long) size);
   return 0;
 }
 
