@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999-2001, 2007, 2010-2012 Free Software Foundation,
+   Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -40,16 +39,16 @@ static const char *with_defs[] =
 int
 mail_version (int argc MU_ARG_UNUSED, char **argv MU_ARG_UNUSED)
 { 
-  fprintf (ofile, "%s", argp_program_version);
+  mu_printf ("%s", program_version);
   if (with_defs[0] != NULL)
     {
       int i;
-      fprintf (ofile, " (");
+      mu_printf (" (");
       for (i = 0; with_defs[i]; i++)
-	fprintf (ofile, " %s", with_defs[i]);
-      fprintf (ofile, " )");
+	mu_printf (" %s", with_defs[i]);
+      mu_printf (" )");
     }
-  fprintf (ofile, "\n");
+  mu_printf ("\n");
 
   return 0;
 }
