@@ -103,8 +103,8 @@
 #line 1 "mh_fmtgram.y"
 
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2004,
-   2007, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1999-2002, 2004, 2007, 2009-2012 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -117,16 +117,14 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <mh.h>
 #define obstack_chunk_alloc malloc
 #define obstack_chunk_free free
 #include <obstack.h>
 
-int fmt_yyerror (char *s);
+int fmt_yyerror (const char *s);
 int fmt_yylex ();
  
 static mh_format_t format;     /* Format structure being built */
@@ -170,7 +168,7 @@ static int want_arg;        /* Expecting function argument */
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 50 "mh_fmtgram.y"
+#line 48 "mh_fmtgram.y"
 {
   char *str;
   int num;
@@ -183,7 +181,7 @@ typedef union YYSTYPE
   mh_builtin_t *builtin;
 }
 /* Line 187 of yacc.c.  */
-#line 187 "mh_fmtgram.c"
+#line 185 "mh_fmtgram.c"
 	YYSTYPE;
 # define fmt_yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
@@ -196,7 +194,7 @@ typedef union YYSTYPE
 
 
 /* Line 216 of yacc.c.  */
-#line 200 "mh_fmtgram.c"
+#line 198 "mh_fmtgram.c"
 
 #ifdef short
 # undef short
@@ -493,11 +491,11 @@ static const fmt_yytype_int8 fmt_yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const fmt_yytype_uint16 fmt_yyrline[] =
 {
-       0,    75,    75,    81,    82,    85,   110,   111,   117,   122,
-     129,   130,   131,   137,   152,   158,   164,   164,   164,   192,
-     193,   200,   201,   217,   220,   221,   225,   258,   261,   264,
-     270,   282,   290,   296,   307,   308,   312,   316,   324,   330,
-     340,   343,   346
+       0,    73,    73,    79,    80,    83,   108,   109,   115,   120,
+     127,   128,   129,   135,   150,   156,   162,   162,   162,   190,
+     191,   198,   199,   215,   218,   219,   223,   256,   259,   262,
+     268,   280,   288,   294,   305,   306,   310,   314,   322,   328,
+     338,   341,   344
 };
 #endif
 
@@ -1439,14 +1437,14 @@ fmt_yyreduce:
   switch (fmt_yyn)
     {
         case 2:
-#line 76 "mh_fmtgram.y"
+#line 74 "mh_fmtgram.y"
     {
 	      /* nothing: to shut bison up */
 	    }
     break;
 
   case 5:
-#line 86 "mh_fmtgram.y"
+#line 84 "mh_fmtgram.y"
     {
 	      switch ((fmt_yyvsp[(1) - (1)].type))
 		{
@@ -1472,14 +1470,14 @@ fmt_yyreduce:
     break;
 
   case 7:
-#line 112 "mh_fmtgram.y"
+#line 110 "mh_fmtgram.y"
     {
 	      in_escape = 0;
 	    }
     break;
 
   case 8:
-#line 118 "mh_fmtgram.y"
+#line 116 "mh_fmtgram.y"
     {
 	      mh_code_string ((fmt_yyvsp[(1) - (1)].str));
 	      (fmt_yyval.type) = mhtype_str;
@@ -1487,7 +1485,7 @@ fmt_yyreduce:
     break;
 
   case 9:
-#line 123 "mh_fmtgram.y"
+#line 121 "mh_fmtgram.y"
     {
 	      mh_code_number ((fmt_yyvsp[(1) - (1)].num));
 	      (fmt_yyval.type) = mhtype_num;
@@ -1495,14 +1493,14 @@ fmt_yyreduce:
     break;
 
   case 12:
-#line 132 "mh_fmtgram.y"
+#line 130 "mh_fmtgram.y"
     {
 	      (fmt_yyval.type) = mhtype_none;
 	    }
     break;
 
   case 13:
-#line 138 "mh_fmtgram.y"
+#line 136 "mh_fmtgram.y"
     {
 	      if (mu_c_strcasecmp ((fmt_yyvsp[(3) - (4)].str), "body") == 0)
 		{
@@ -1518,31 +1516,31 @@ fmt_yyreduce:
     break;
 
   case 14:
-#line 153 "mh_fmtgram.y"
+#line 151 "mh_fmtgram.y"
     {
 	      in_escape++;
 	    }
     break;
 
   case 15:
-#line 159 "mh_fmtgram.y"
+#line 157 "mh_fmtgram.y"
     {
 	      in_escape--;
 	    }
     break;
 
   case 16:
-#line 164 "mh_fmtgram.y"
+#line 162 "mh_fmtgram.y"
     { want_function = 1;}
     break;
 
   case 17:
-#line 164 "mh_fmtgram.y"
+#line 162 "mh_fmtgram.y"
     { want_function = 0; want_arg = 1;}
     break;
 
   case 18:
-#line 165 "mh_fmtgram.y"
+#line 163 "mh_fmtgram.y"
     {
 	      if ((fmt_yyvsp[(4) - (7)].builtin))
 		{
@@ -1571,7 +1569,7 @@ fmt_yyreduce:
     break;
 
   case 20:
-#line 194 "mh_fmtgram.y"
+#line 192 "mh_fmtgram.y"
     {
 	      mh_code_op (mhop_fmtspec);
 	      mh_code_op ((fmt_yyvsp[(1) - (1)].num));
@@ -1579,7 +1577,7 @@ fmt_yyreduce:
     break;
 
   case 22:
-#line 202 "mh_fmtgram.y"
+#line 200 "mh_fmtgram.y"
     {
 	      if (strcmp ((fmt_yyvsp[(1) - (1)].str), "void") == 0)
 		{
@@ -1588,21 +1586,21 @@ fmt_yyreduce:
 	      else
 		{
 		  fmt_yyerror (_("undefined function"));
-		  mu_error ((fmt_yyvsp[(1) - (1)].str));
+		  mu_error ("%s", (fmt_yyvsp[(1) - (1)].str));
 		  YYERROR;
 		}
 	    }
     break;
 
   case 23:
-#line 217 "mh_fmtgram.y"
+#line 215 "mh_fmtgram.y"
     {
 	      (fmt_yyval.type) = mhtype_none;
 	    }
     break;
 
   case 26:
-#line 226 "mh_fmtgram.y"
+#line 224 "mh_fmtgram.y"
     {
 	      size_t start_pc = 0, end_pc = 0;
 
@@ -1635,21 +1633,21 @@ fmt_yyreduce:
     break;
 
   case 27:
-#line 258 "mh_fmtgram.y"
+#line 256 "mh_fmtgram.y"
     {
 	      (fmt_yyval.pc) = pc;
 	    }
     break;
 
   case 29:
-#line 265 "mh_fmtgram.y"
+#line 263 "mh_fmtgram.y"
     {
 	      in_escape++;
 	    }
     break;
 
   case 30:
-#line 271 "mh_fmtgram.y"
+#line 269 "mh_fmtgram.y"
     {
 	      /* False branch of an if-block */
 	      (fmt_yyval.elif_list).cond = mh_code_op (mhop_num_asgn);
@@ -1662,7 +1660,7 @@ fmt_yyreduce:
     break;
 
   case 31:
-#line 283 "mh_fmtgram.y"
+#line 281 "mh_fmtgram.y"
     {
 	      in_escape++;
 	      (fmt_yyval.pc) = pc;
@@ -1670,7 +1668,7 @@ fmt_yyreduce:
     break;
 
   case 32:
-#line 290 "mh_fmtgram.y"
+#line 288 "mh_fmtgram.y"
     {
 	      mh_code_op (mhop_branch);
 	      (fmt_yyval.pc) = mh_code_op (0);
@@ -1678,7 +1676,7 @@ fmt_yyreduce:
     break;
 
   case 33:
-#line 297 "mh_fmtgram.y"
+#line 295 "mh_fmtgram.y"
     {
 	      in_escape--;
 	      if ((fmt_yyvsp[(1) - (1)].type) == mhtype_str)
@@ -1690,7 +1688,7 @@ fmt_yyreduce:
     break;
 
   case 36:
-#line 312 "mh_fmtgram.y"
+#line 310 "mh_fmtgram.y"
     {
 	      (fmt_yyval.elif_list).cond = 0;
 	      (fmt_yyval.elif_list).end = 0;
@@ -1698,7 +1696,7 @@ fmt_yyreduce:
     break;
 
   case 37:
-#line 317 "mh_fmtgram.y"
+#line 315 "mh_fmtgram.y"
     {
 	      (fmt_yyval.elif_list).cond = (fmt_yyvsp[(1) - (2)].elif_list).cond;
 	      MHI_NUM(format.prog[(fmt_yyvsp[(2) - (2)].pc)]) = (fmt_yyvsp[(1) - (2)].elif_list).end;
@@ -1707,7 +1705,7 @@ fmt_yyreduce:
     break;
 
   case 38:
-#line 325 "mh_fmtgram.y"
+#line 323 "mh_fmtgram.y"
     {
 	      (fmt_yyval.elif_list).cond = (fmt_yyvsp[(1) - (3)].pc);
 	      MHI_NUM(format.prog[(fmt_yyvsp[(2) - (3)].pc)]) = pc - (fmt_yyvsp[(2) - (3)].pc) + 2;
@@ -1716,7 +1714,7 @@ fmt_yyreduce:
     break;
 
   case 39:
-#line 331 "mh_fmtgram.y"
+#line 329 "mh_fmtgram.y"
     {
 	      MHI_NUM(format.prog[(fmt_yyvsp[(4) - (5)].pc)]) = pc - (fmt_yyvsp[(4) - (5)].pc) + 2;
 	      (fmt_yyval.elif_list).cond = (fmt_yyvsp[(1) - (5)].elif_list).cond;
@@ -1726,14 +1724,14 @@ fmt_yyreduce:
     break;
 
   case 40:
-#line 340 "mh_fmtgram.y"
+#line 338 "mh_fmtgram.y"
     {
 	      (fmt_yyval.pc) = 0;
 	    }
     break;
 
   case 42:
-#line 347 "mh_fmtgram.y"
+#line 345 "mh_fmtgram.y"
     {
 	      (fmt_yyval.pc) = pc;
 	    }
@@ -1741,7 +1739,7 @@ fmt_yyreduce:
 
 
 /* Line 1267 of yacc.c.  */
-#line 1745 "mh_fmtgram.c"
+#line 1743 "mh_fmtgram.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", fmt_yyr1[fmt_yyn], &fmt_yyval, &fmt_yyloc);
@@ -1955,14 +1953,14 @@ fmt_yyreturn:
 }
 
 
-#line 352 "mh_fmtgram.y"
+#line 350 "mh_fmtgram.y"
 
 
 static char *start;
 static char *curp;
 
 int
-fmt_yyerror (char *s)
+fmt_yyerror (const char *s)
 {
   int len;
   mu_error ("%s: %s", start, s);
