@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2007, 2009,
-   2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 1999-2003, 2005-2007, 2009-2012 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -454,7 +454,7 @@ addr_cmp (void *item, void *data)
 static int
 addrlist_lookup (mu_list_t list, mu_address_t addr)
 {
-  return mu_list_do (list, addr_cmp, addr);
+  return mu_list_foreach (list, addr_cmp, addr);
 }
 
 static int
@@ -468,7 +468,7 @@ addr_free (void *item, void *data)
 static void
 addrlist_destroy (mu_list_t *list)
 {
-  mu_list_do (*list, addr_free, NULL);
+  mu_list_foreach (*list, addr_free, NULL);
   mu_list_destroy (list);
 }
 
@@ -1160,7 +1160,7 @@ builtin_putnumf (struct mh_machine *mach)
 }
 
 static int
-_parse_date (struct mh_machine *mach, struct tm *tm, mu_timezone *tz)
+_parse_date (struct mh_machine *mach, struct tm *tm, struct mu_timezone *tz)
 {
   char *date = strobj_ptr (&mach->arg_str);
   const char *p = date;
@@ -1172,7 +1172,7 @@ _parse_date (struct mh_machine *mach, struct tm *tm, mu_timezone *tz)
       /*mu_error ("can't parse date: [%s]", date);*/
       time (&t);
       *tm = *localtime (&t);
-      tz->utc_offset = mu_utc_offset ();
+      mu_datetime_tz_local (tz);
     }
   
   return 0;
@@ -1183,7 +1183,7 @@ static void
 builtin_sec (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   
   if (_parse_date (mach, &tm, &tz))
     return;
@@ -1196,7 +1196,7 @@ static void
 builtin_min (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   
   if (_parse_date (mach, &tm, &tz))
     return;
@@ -1209,7 +1209,7 @@ static void
 builtin_hour (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   
   if (_parse_date (mach, &tm, &tz))
     return;
@@ -1222,7 +1222,7 @@ static void
 builtin_wday (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   
   if (_parse_date (mach, &tm, &tz))
     return;
@@ -1235,7 +1235,7 @@ static void
 builtin_day (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   char buf[80];
   
   if (_parse_date (mach, &tm, &tz))
@@ -1251,7 +1251,7 @@ static void
 builtin_weekday (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   char buf[80];
   
   if (_parse_date (mach, &tm, &tz))
@@ -1268,7 +1268,7 @@ static void
 builtin_sday (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
 
   /*FIXME: more elaborate check needed */
   if (_parse_date (mach, &tm, &tz))
@@ -1282,7 +1282,7 @@ static void
 builtin_mday (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   
   if (_parse_date (mach, &tm, &tz))
     return;
@@ -1295,7 +1295,7 @@ static void
 builtin_yday (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   
   if (_parse_date (mach, &tm, &tz))
     return;
@@ -1308,7 +1308,7 @@ static void
 builtin_mon (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   
   if (_parse_date (mach, &tm, &tz))
     return;
@@ -1321,7 +1321,7 @@ static void
 builtin_month (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   char buf[80];
   
   if (_parse_date (mach, &tm, &tz))
@@ -1337,7 +1337,7 @@ static void
 builtin_lmonth (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   char buf[80];
   
   if (_parse_date (mach, &tm, &tz))
@@ -1353,7 +1353,7 @@ static void
 builtin_year (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   
   if (_parse_date (mach, &tm, &tz))
     return;
@@ -1366,7 +1366,7 @@ static void
 builtin_zone (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   
   if (_parse_date (mach, &tm, &tz))
     return;
@@ -1379,7 +1379,7 @@ static void
 builtin_tzone (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   
   if (_parse_date (mach, &tm, &tz))
     return;
@@ -1410,7 +1410,7 @@ static void
 builtin_szone (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
 
   /*FIXME: more elaborate check needed */
   if (_parse_date (mach, &tm, &tz))
@@ -1438,7 +1438,7 @@ static void
 builtin_dst (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
 
   if (_parse_date (mach, &tm, &tz))
     return;
@@ -1454,11 +1454,11 @@ static void
 builtin_clock (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
 
   if (_parse_date (mach, &tm, &tz))
     return;
-  mach->arg_num = mu_tm2time (&tm, &tz);
+  mach->arg_num = mu_datetime_to_utc (&tm, &tz);
 }
 
 /*     rclock     date     integer  seconds prior to current time*/
@@ -1466,12 +1466,12 @@ void
 builtin_rclock (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   time_t now = time (NULL);
   
   if (_parse_date (mach, &tm, &tz))
     return;
-  mach->arg_num = now - mu_tm2time (&tm, &tz);
+  mach->arg_num = now - mu_datetime_to_utc (&tm, &tz);
 }
 
 struct
@@ -1493,7 +1493,7 @@ static void
 date_cvt (struct mh_machine *mach, int pretty)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   char buf[80];
   int i, len;
   const char *tzname = NULL;
@@ -1567,7 +1567,7 @@ static void
 builtin_nodate (struct mh_machine *mach)
 {
   struct tm tm;
-  mu_timezone tz;
+  struct mu_timezone tz;
   
   mach->arg_num = _parse_date (mach, &tm, &tz);
 }
@@ -1801,7 +1801,6 @@ static void
 builtin_formataddr (struct mh_machine *mach)
 {
   mu_address_t addr, dest;
-  size_t size;
   int i;
   size_t num;
   const char *buf;
@@ -1837,12 +1836,12 @@ builtin_formataddr (struct mh_machine *mach)
 	}
     }
 
-  if (mu_address_to_string (dest, NULL, 0, &size) == 0)
+  if (mu_address_sget_printable (dest, &buf) == 0)
     {
-      strobj_realloc (&mach->reg_str, size + 1);
-      mu_address_to_string (dest, strobj_ptr (&mach->reg_str), size + 1, NULL);
-      mu_address_destroy (&dest);
+      strobj_realloc (&mach->reg_str, strlen (buf) + 1);
+      strcpy (strobj_ptr (&mach->reg_str), buf);
     }
+  mu_address_destroy (&dest);
 }
 
 /*      putaddr    literal        print str address list with
