@@ -87,8 +87,7 @@
 #line 1 "mimetypes.y"
 
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2005, 2007, 2009, 2010, 2011 Free Software Foundation,
-   Inc.
+   Copyright (C) 2005, 2007, 2009-2012 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -213,7 +212,7 @@ static mu_list_t rule_list;
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 120 "mimetypes.y"
+#line 119 "mimetypes.y"
 {
   struct mimetypes_string string;
   mu_list_t list;
@@ -221,7 +220,7 @@ typedef union YYSTYPE
   struct node *node;
 }
 /* Line 187 of yacc.c.  */
-#line 225 "mimetypes-gram.c"
+#line 224 "mimetypes-gram.c"
 	YYSTYPE;
 # define mimetypes_yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
@@ -234,7 +233,7 @@ typedef union YYSTYPE
 
 
 /* Line 216 of yacc.c.  */
-#line 238 "mimetypes-gram.c"
+#line 237 "mimetypes-gram.c"
 
 #ifdef short
 # undef short
@@ -524,9 +523,9 @@ static const mimetypes_yytype_int8 mimetypes_yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const mimetypes_yytype_uint8 mimetypes_yyrline[] =
 {
-       0,   129,   129,   132,   133,   136,   137,   146,   155,   156,
-     159,   165,   166,   170,   174,   180,   184,   188,   192,   195,
-     196,   199,   208,   214,   221
+       0,   128,   128,   131,   132,   135,   136,   145,   154,   155,
+     158,   164,   165,   169,   173,   179,   183,   187,   191,   194,
+     195,   198,   207,   213,   220
 };
 #endif
 
@@ -1451,7 +1450,7 @@ mimetypes_yyreduce:
   switch (mimetypes_yyn)
     {
         case 6:
-#line 138 "mimetypes.y"
+#line 137 "mimetypes.y"
     {
 	     struct rule_tab *p = mimetypes_malloc (sizeof (*p));
 	     if (!rule_list)
@@ -1463,7 +1462,7 @@ mimetypes_yyreduce:
     break;
 
   case 7:
-#line 147 "mimetypes.y"
+#line 146 "mimetypes.y"
     {
 	     if (arg_list)
 	       mu_list_destroy (&arg_list);
@@ -1473,56 +1472,56 @@ mimetypes_yyreduce:
     break;
 
   case 10:
-#line 160 "mimetypes.y"
+#line 159 "mimetypes.y"
     {
 	     (mimetypes_yyval.string) = mimetypes_append_string2 (&(mimetypes_yyvsp[(1) - (3)].string), '/', &(mimetypes_yyvsp[(3) - (3)].string));
 	   }
     break;
 
   case 12:
-#line 167 "mimetypes.y"
+#line 166 "mimetypes.y"
     {
 	     (mimetypes_yyval.node) = make_binary_node (L_OR, (mimetypes_yyvsp[(1) - (2)].node), (mimetypes_yyvsp[(2) - (2)].node));
 	   }
     break;
 
   case 13:
-#line 171 "mimetypes.y"
+#line 170 "mimetypes.y"
     {
 	     (mimetypes_yyval.node) = make_binary_node (L_OR, (mimetypes_yyvsp[(1) - (3)].node), (mimetypes_yyvsp[(3) - (3)].node));
 	   }
     break;
 
   case 14:
-#line 175 "mimetypes.y"
+#line 174 "mimetypes.y"
     {
 	     (mimetypes_yyval.node) = make_binary_node (L_AND, (mimetypes_yyvsp[(1) - (3)].node), (mimetypes_yyvsp[(3) - (3)].node));
 	   }
     break;
 
   case 15:
-#line 181 "mimetypes.y"
+#line 180 "mimetypes.y"
     {
 	     (mimetypes_yyval.node) = make_negation_node ((mimetypes_yyvsp[(2) - (2)].node));
 	   }
     break;
 
   case 16:
-#line 185 "mimetypes.y"
+#line 184 "mimetypes.y"
     {
 	     (mimetypes_yyval.node) = (mimetypes_yyvsp[(2) - (3)].node);
 	   }
     break;
 
   case 17:
-#line 189 "mimetypes.y"
+#line 188 "mimetypes.y"
     {
 	     (mimetypes_yyval.node) = make_suffix_node (&(mimetypes_yyvsp[(1) - (1)].string));
 	   }
     break;
 
   case 21:
-#line 200 "mimetypes.y"
+#line 199 "mimetypes.y"
     {
 	     reset_lex ();
 	     (mimetypes_yyval.node) = make_functional_node ((mimetypes_yyvsp[(1) - (3)].string).ptr, (mimetypes_yyvsp[(2) - (3)].list));
@@ -1532,7 +1531,7 @@ mimetypes_yyreduce:
     break;
 
   case 22:
-#line 209 "mimetypes.y"
+#line 208 "mimetypes.y"
     {
 	     mu_list_create (&arg_list);
 	     (mimetypes_yyval.list) = arg_list;
@@ -1541,7 +1540,7 @@ mimetypes_yyreduce:
     break;
 
   case 23:
-#line 215 "mimetypes.y"
+#line 214 "mimetypes.y"
     {
 	     mu_list_append ((mimetypes_yyvsp[(1) - (3)].list), mimetypes_string_dup (&(mimetypes_yyvsp[(3) - (3)].string)));
 	     (mimetypes_yyval.list) = (mimetypes_yyvsp[(1) - (3)].list);
@@ -1550,7 +1549,7 @@ mimetypes_yyreduce:
 
 
 /* Line 1267 of yacc.c.  */
-#line 1554 "mimetypes-gram.c"
+#line 1553 "mimetypes-gram.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", mimetypes_yyr1[mimetypes_yyn], &mimetypes_yyval, &mimetypes_yyloc);
@@ -1764,7 +1763,7 @@ mimetypes_yyreturn:
 }
 
 
-#line 224 "mimetypes.y"
+#line 223 "mimetypes.y"
 
 
 int
@@ -2246,7 +2245,7 @@ const char *
 get_file_type ()
 {
   const char *type = NULL;
-  mu_list_do (rule_list, evaluate, &type);
+  mu_list_foreach (rule_list, evaluate, &type);
   return type;
 }
     
