@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2007,
-   2009 Free Software Foundation, Inc.
+   Copyright (C) 1999-2002, 2005, 2007, 2009-2012 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -41,104 +41,121 @@
 char *
 pop3d_apopuser (const char *user)
 {
-  char *password;
-  char buf[POP_MAXCMDLEN];
-
-#ifdef USE_DBM
+  char *password = NULL;
+  int rc;
+  
+#ifdef ENABLE_DBM
   {
-    DBM_FILE db;
-    DBM_DATUM key, data;
+    size_t len;
+    mu_dbm_file_t db;
+    struct mu_dbm_datum key, data;
 
-    int rc = mu_dbm_open (APOP_PASSFILE, &db, MU_STREAM_READ, 0600);
+    rc = mu_dbm_create (apop_database_name, &db, apop_database_safety);
+    if (rc)
+      {
+	mu_diag_output (MU_DIAG_ERROR, _("unable to create APOP db"));
+	return NULL;
+      }
+
+    if (apop_database_owner_set)
+      mu_dbm_safety_set_owner (db, apop_database_owner);
+    
+    rc = mu_dbm_safety_check (db);
+    if (rc)
+      {
+	mu_diag_output (MU_DIAG_ERROR,
+			_("APOP file %s fails safety check: %s"),
+			apop_database_name, mu_strerror (rc));
+	mu_dbm_destroy (&db);
+	return NULL;
+      }
+    
+    rc = mu_dbm_open (db, MU_STREAM_READ, 0600);
     if (rc)
       {
 	mu_diag_output (MU_DIAG_ERROR, _("unable to open APOP db: %s"),
-		mu_strerror (errno));
+			mu_strerror (rc));
 	return NULL;
       }
 
     memset (&key, 0, sizeof key);
     memset (&data, 0, sizeof data);
 
-    strncpy (buf, user, sizeof buf);
-    /* strncpy () is lame and does not NULL terminate.  */
-    buf[sizeof (buf) - 1] = '\0';
-    MU_DATUM_PTR(key) = buf;
-    MU_DATUM_SIZE(key) = strlen (buf);
+    key.mu_dptr = (void *)user;
+    key.mu_dsize = strlen (user);
 
-    rc = mu_dbm_fetch (db, key, &data);
-    mu_dbm_close (db);
-    if (rc)
+    rc = mu_dbm_fetch (db, &key, &data);
+    if (rc == MU_ERR_NOENT)
       {
-	mu_diag_output (MU_DIAG_ERROR,
-			_("cannot fetch APOP data: %s"), mu_strerror (errno));
+	mu_dbm_destroy (&db);
 	return NULL;
       }
-    password = calloc (MU_DATUM_SIZE(data) + 1, sizeof (*password));
+    else if (rc)
+      {
+	mu_diag_output (MU_DIAG_ERROR,
+			_("cannot fetch APOP data: %s"),
+			mu_dbm_strerror (db));
+	mu_dbm_destroy (&db);
+	return NULL;
+      }
+    mu_dbm_destroy (&db);
+    len = data.mu_dsize;
+    password = malloc (len + 1);
     if (password == NULL)
       {
 	mu_dbm_datum_free (&data);
 	return NULL;
       }
-    
-    sprintf (password, "%.*s", (int) MU_DATUM_SIZE(data),
-	     (char*) MU_DATUM_PTR(data));
+    memcpy (password, data.mu_dptr, len);
+    password[len] = 0;
     mu_dbm_datum_free (&data);
     return password;
   }
-#else /* !USE_DBM */
+#else /* !ENABLE_DBM */
   {
-    char *tmp;
+    char *buf = NULL;
+    size_t size = 0;
+    size_t ulen;
     FILE *apop_file;
 
-    if (mu_check_perm (APOP_PASSFILE, 0600))
+    rc = mu_file_safety_check (apop_database_name, apop_database_safety,
+			       apop_database_owner, NULL);
+    if (rc)
       {
-	mu_diag_output (MU_DIAG_INFO,
-			_("bad permissions on APOP password file"));
+	mu_diag_output (MU_DIAG_ERROR,
+			_("APOP file %s fails safety check: %s"),
+			apop_database_name, mu_strerror (rc));
 	return NULL;
-    }
-
-    apop_file = fopen (APOP_PASSFILE, "r");
+      }
+    apop_file = fopen (apop_database_name, "r");
     if (apop_file == NULL)
       {
-	mu_diag_output (MU_DIAG_INFO, _("unable to open APOP password file %s"),
-		strerror (errno));
+	mu_diag_output (MU_DIAG_INFO,
+			_("unable to open APOP password file %s: %s"),
+			apop_database_name, mu_strerror (errno));
 	return NULL;
       }
 
-    password = calloc (APOP_DIGEST, sizeof (*password));
-    if (password == NULL)
+    ulen = strlen (user);
+    while (getline (&buf, &size, apop_file) > 0)
       {
-	fclose (apop_file);
-	pop3d_abquit (ERR_NO_MEM);
-      }
+	char *p, *start = mu_str_stripws (buf);
 
-    while (fgets (buf, sizeof (buf) - 1, apop_file) != NULL)
-      {
-	tmp = strchr (buf, ':');
-	if (tmp == NULL)
+	if (!*start || *start == '#')
 	  continue;
-	*tmp++ = '\0';
-
-	if (strncmp (user, buf, strlen (user)))
+	p = strchr (start, ':');
+	if (!p)
 	  continue;
-
-	strncpy (password, tmp, APOP_DIGEST);
-	/* strncpy () is lame and does not NULL terminate.  */
-	password[APOP_DIGEST - 1] = '\0';
-	tmp = strchr (password, '\n');
-	if (tmp)
-	  *tmp = '\0';
-	break;
+	if (p - start == ulen && memcmp (user, start, ulen) == 0)
+	  {
+	    p = mu_str_skip_class (p + 1, MU_CTYPE_SPACE);
+	    if (*p)
+	      password = strdup (p);
+	    break;
+	  }
       }
 
     fclose (apop_file);
-    if (*password == '\0')
-      {
-	free (password);
-	return NULL;
-      }
-
     return password;
   }
 #endif
@@ -147,11 +164,12 @@ pop3d_apopuser (const char *user)
 int
 pop3d_apop (char *arg)
 {
-  char *tmp, *password, *user_digest, *user;
-  char buf[POP_MAXCMDLEN];
+  char *p, *password, *user_digest, *user;
   struct mu_md5_ctx md5context;
   unsigned char md5digest[16];
-
+  char buf[2 * 16 + 1];
+  size_t i;
+  
   if (state != AUTHORIZATION)
     return ERR_WRONG_STATE;
 
@@ -159,11 +177,6 @@ pop3d_apop (char *arg)
     return ERR_BAD_ARGS;
 
   pop3d_parse_command (arg, &user, &user_digest);
-  if (strlen (user) > (POP_MAXCMDLEN - APOP_DIGEST))
-    {
-      mu_diag_output (MU_DIAG_INFO, _("user name too long: %s"), user);
-      return ERR_BAD_ARGS;
-    }
 
   password = pop3d_apopuser (user);
   if (password == NULL)
@@ -180,14 +193,9 @@ pop3d_apop (char *arg)
   free (password);
   mu_md5_finish_ctx (&md5context, md5digest);
 
-  {
-    int i;
-    tmp = buf;
-    for (i = 0; i < 16; i++, tmp += 2)
-      sprintf (tmp, "%02x", md5digest[i]);
-  }
-
-  *tmp++ = '\0';
+  for (i = 0, p = buf; i < 16; i++, p += 2)
+      sprintf (p, "%02x", md5digest[i]);
+  *p = 0;
 
   if (strcmp (user_digest, buf))
     {
