@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008,
-   2009, 2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 1999-2005, 2007-2012 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -33,8 +32,9 @@ int debug_mode;
 int tls_required;
 int pop3d_xlines;
 char *apop_database_name = APOP_PASSFILE;
-int apop_database_safety;
-int apop_database_safety_set;
+int apop_database_safety = MU_FILE_SAFETY_ALL;
+uid_t apop_database_owner;
+int apop_database_owner_set;
 
 #ifdef WITH_TLS
 int tls_available;
@@ -92,11 +92,37 @@ cb2_file_safety_checks (const char *name, void *data)
 static int
 cb_apop_safety_checks (void *data, mu_config_value_t *arg)
 {
-  apop_database_safety_set = 1;
   return mu_cfg_string_value_cb (arg, cb2_file_safety_checks,
 				 &apop_database_safety);
 }
 
+static int
+cb_apop_database_owner (void *data, mu_config_value_t *val)
+{
+  struct passwd *pw;
+  
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
+    return 1;
+  pw = getpwnam (val->v.string);
+  if (!pw)
+    {
+      char *p;
+      unsigned long n;
+    
+      n = strtoul (val->v.string, &p, 10);
+      if (*p)
+	{
+	  mu_error (_("no such user: %s"), val->v.string);
+	  return 1;
+	}
+      apop_database_owner = n;
+    }
+  else
+    apop_database_owner = pw->pw_uid;
+  apop_database_owner_set = 1;
+  return 0;
+}
+
 #ifdef ENABLE_DBM
 static int
 cb_bulletin_db (void *data, mu_config_value_t *val)
@@ -133,6 +159,8 @@ static struct mu_cfg_param pop3d_cfg_param[] = {
     N_("Output the number of lines in the message in its scan listing.") },
   { "apop-database-file", mu_cfg_string, &apop_database_name, 0, NULL,
     N_("set APOP database file name or URL") },
+  { "apop-database-owner", mu_cfg_callback, NULL, 0, cb_apop_database_owner,
+    N_("Name or UID of the APOP database owner") },
   { "apop-database-safety", mu_cfg_callback, NULL, 0, cb_apop_safety_checks,
     N_("Configure safety checks for APOP database files.  Argument is a list or "
        "sequence of check names optionally prefixed with '+' to enable or "
@@ -338,17 +366,90 @@ pop3d_mainloop (int ifd, int ofd, int tls)
   return status;
 }
 
+static int
+set_strerr_flt ()
+{
+  mu_stream_t flt, trans[2];
+  int rc;
+  
+  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_SUBSTREAM, MU_IOCTL_OP_GET, trans);
+  if (rc == 0)
+    {
+      char sessidstr[10];
+      char *argv[] = { "inline-comment", NULL, "-S", NULL };
+
+      snprintf (sessidstr, sizeof sessidstr, "%08lx:", mu_session_id);
+      argv[1] = sessidstr;
+      rc = mu_filter_create_args (&flt, trans[0], "inline-comment", 3,
+				  (const char **)argv,
+				  MU_FILTER_ENCODE, MU_STREAM_WRITE);
+      mu_stream_unref (trans[0]);
+      if (rc == 0)
+	{
+	  mu_stream_set_buffer (flt, mu_buffer_line, 0);
+	  trans[0] = flt;
+	  trans[1] = NULL;
+	  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_SUBSTREAM,
+				MU_IOCTL_OP_SET, trans);
+	  mu_stream_unref (trans[0]);
+	  if (rc)
+	    mu_error (_("%s failed: %s"), "MU_IOCTL_SET_STREAM",
+		      mu_stream_strerror (mu_strerr, rc));
+	}
+      else
+	mu_error (_("cannot create log filter stream: %s"), mu_strerror (rc));
+    }
+  else
+    {
+      mu_error (_("%s failed: %s"), "MU_IOCTL_GET_STREAM",
+		mu_stream_strerror (mu_strerr, rc));
+    }
+  return rc;
+}
+
+static void
+clr_strerr_flt ()
+{
+  mu_stream_t flt, trans[2];
+  int rc;
+
+  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_SUBSTREAM, MU_IOCTL_OP_GET, trans);
+  if (rc == 0)
+    {
+      flt = trans[0];
+
+      rc = mu_stream_ioctl (flt, MU_IOCTL_SUBSTREAM, MU_IOCTL_OP_GET, trans);
+      if (rc == 0)
+	{
+	  mu_stream_unref (trans[0]);
+	  rc = mu_stream_ioctl (mu_strerr, MU_IOCTL_SUBSTREAM,
+				MU_IOCTL_OP_SET, trans);
+	  if (rc == 0)
+	    mu_stream_unref (flt);
+	}
+    }
+}
+
 int
 pop3d_connection (int fd, struct sockaddr *sa, int salen,
 		  struct mu_srv_config *pconf,
 		  void *data)
 {
   struct pop3d_srv_config *cfg = (struct pop3d_srv_config *) pconf;
+  int rc;
   
   idle_timeout = pconf->timeout;
   pop3d_transcript = pconf->transcript;
 
+  if (mu_log_session_id)
+    rc = set_strerr_flt ();
+  else
+    rc = 1;
+  
   pop3d_mainloop (fd, fd, cfg->tls);
+
+  if (rc == 0)
+    clr_strerr_flt ();
   return 0;
 }
 
